var path = require('path');

module.exports = function (api, externalOptions) {
  api.assertVersion(7);
  var options = {
    target: externalOptions.target || 'js',
    environments: externalOptions.environments || {
      node: 'current',
      browsers: [// https://gitlab.skypicker.com/frontend/frontend/blob/master/webpack.common.js
      'last 2 versions', 'ie >= 11']
    },
    debug: externalOptions.debug || false
  };
  var presets = [];
  var plugins = [path.join(__dirname, 'dev-expression-check.js')];
  var parserPlugins = ['jsx', 'flow', 'flowComments', 'bigInt', // https://github.com/tc39/proposal-bigint
  'throwExpressions' // https://github.com/tc39/proposal-throw-expressions
  ];
  var retainLines = false;
  var target = options.target;

  if (target === 'flow') {
    plugins = plugins.concat([path.join(__dirname, 'dev-declaration.js')]);
    parserPlugins = parserPlugins.concat([// These parser options are relevant only to Flow because JS targets
    // enable them via necessary transpilation plugins.
    'classPrivateProperties', // https://github.com/tc39/proposal-private-fields
    'classProperties', // https://github.com/tc39/proposal-class-public-fields
    'numericSeparator', // https://github.com/tc39/proposal-numeric-separator
    'objectRestSpread' // https://github.com/tc39/proposal-object-rest-spread
    ]);
    retainLines = true;
  } else if (target === 'js' || target === 'js-esm') {
    var supportsESM = target === 'js-esm';
    presets = presets.concat([['@babel/preset-env', {
      debug: options.debug,
      modules: supportsESM ? false : 'commonjs',
      targets: options.environments,
      bugfixes: true // TODO - loose: true (?)

    }], '@babel/preset-react']);
    plugins = plugins.concat([path.join(__dirname, 'dev-expression.js'), path.join(__dirname, 'adeira-js-invariant.js'), path.join(__dirname, 'adeira-js-warning.js'), '@babel/plugin-transform-flow-strip-types', // https://github.com/babel/babel/issues/8417
    '@babel/plugin-proposal-class-properties', '@babel/plugin-proposal-object-rest-spread', '@babel/plugin-proposal-numeric-separator', // Transform runtime plugin turns common chunks of code into imports. However, this
    // requires `@babel/runtime` dependency thus we are requiring it as well.
    // See: https://babeljs.io/docs/en/babel-plugin-transform-runtime
    ['@babel/plugin-transform-runtime', {
      useESModules: supportsESM
    }]]);
  } else {
    target;
    throw new Error('options.target must be one of "js" or "flow".');
  }

  return {
    presets: presets,
    plugins: plugins,
    parserOpts: {
      // see: https://babeljs.io/docs/en/babel-parser#plugins
      // Candidates: classPrivateMethods, v8intrinsic (__DEV__)
      plugins: parserPlugins
    },
    retainLines: retainLines // TODO: sourceType: 'unambiguous',

  };
};